#!/usr/bin/perl

#
# Script Perl que realiza o processamento de arquivos etiquetados do corpus,
# adaptando-os para utilização nas buscas online. A função deste script é
# inserir ao final de cada parágrafo, a identificação do arquivo, a página e
# a sentença, e uma marca de final de parágrafo. Exemplo:
#
# AIRES-00,240.3/ID ./PONFP
#
# Com isto, o arquivo de saída fica adequado para a busca web do Corpus.
#
# Autor: Pablo Picasso F. de Faria
# Criação: 24/09/2007
# Última alteração: 22/11/2007
#

# Verifica parâmetros
if(scalar(@ARGV) != 1) {
   print "\nScripts do Corpus Tycho Brahe, IEL/Unicamp (2008)\n";
   print "Autor: Pablo Faria (pablofaria\@gmail.com)\n\n";
   print "Uso:\n    tb-adjustPOS.pl <posfile>\n";
   print "Saída:\n    <posfile>.cs\n\n";
   exit;
}

print STDOUT "   Início...\n";

$fin  = $ARGV[0];
$fout = $ARGV[0].".cs";

# Monta a string de identificação do arquivo: aires_00_tcc.txt -> AIRES-00
@vet1   = split('_',$fin);  # Separa os elementos do nome, separados por '_'
$fileid = "";
for ($ii = 0; $ii < (scalar(@vet1) - 2); $ii++) {
    $fileid .= @vet1[$ii]."-";
}
$fileid .= @vet1[$ii];
$fileid =~ tr/[a-z]/[A-Z]/; # Converte para maiúsculas

open(ARQ, "<$fin") || die;
open(DADOS, ">$fout") || die;

$page     = 0;
$pageant  = -1;
$sentence = 1;
$fgtext   = 0; # marca início/fim do contexto de texto etiquetado
$fgcomm   = 0; # marca início/fim de contexto de comentário
$fgorig   = 0; # marca início/fim de contexto de original

print STDOUT "   Preparando o texto para processamento...\n";

# Varre o arquivo, concatenando as linhas em um buffer único
$newoutput = "";
while($linha = <ARQ>) {
   $linha =~ s/\s+$//g;
   if($linha =~ /[\w\d\W]/) {
      $newoutput .= $linha."\n";
   }
}

# Elimina material de possíveis pré-execuções da rotina (IDs e PONFPs)
$newoutput =~ s/( *?[^\/ ]*?\/ID)//g;
$newoutput =~ s/(\.\/PONFP)//g;

# Começa retirando os saltos de linha e fazendo a contração de espaços múltiplos entre elementos, para um só espaço
$newoutput =~ s/([^\.!\?;:])\n/$1 \@\/.\n /g; # Fim de linha FALSO, apenas para não perder as quebras sem ponto final
$newoutput =~ s/([\.!\?;:]) +\@\/\./$1/g;

$newoutput =~ s/POS_0 \@\/\./POS_0/g;
$newoutput =~ s/<text> \@\/\./<text>/g;
$newoutput =~ s/>\/CODE \@\/\./>\/CODE/g;
$newoutput =~ s/\n/ /g;
$newoutput =~ s/ +/ /g;
$newoutput =~ s/|//g; # Elimina marcas de fim de linha (E-Dictor) que vierem incorretamente

# Elimina coisas entre o FORMAT=POS_0 e o <text>
$newoutput =~ s/(=POS_0).*(<text>)/$1 $2/;

# Altera formato de tag XML: de </tag> para <_tag> e volta apenas <_text> para </text>
$newoutput =~ s/<\/([^>]*)>/<_$1>/g;
$newoutput =~ s/<_text>/<\/text>/g;

$newoutput =~ s/<P_([I0-9]*)>\/[^ ]* /<P_$1>\/CODE /g; # "<P_999>/???" -> <P_999>/CODE

# Retira marcas de salto de linha em XML
$newoutput =~ s/<nl>\/CODE//g;

# Elimina comentários
while ($newoutput =~ / *<comment *[^>]*>\/CODE(.*?)<_comment>\/CODE */) {
   # Insere na primeira ocorrência (na sentença) de ponto final
   $pagetext = $1;

   $lastpg = "";
   while ($pagetext =~ /(<P_[0-9]*>\/CODE)/) {
      # Se houverem marcas de página, quando sair, ficará com a última
      $lastpg   = $1;
      $pagetext =~ s/$lastpg//;
   }

   # Insere a página e elimina o comentário
   $newoutput =~ s/( *<comment *[^>]*>\/CODE(.*?)<_comment>\/CODE *)/ $lastpg /;
}

# Elimina adaptações dos originais
while ($newoutput =~ / *<original *[^>]*>\/CODE(.*?)<_original>\/CODE */) {
   # Insere na primeira ocorrência (na sentença) de ponto final
   $pagetext = $1;

   $lastpg = "";
   while ($pagetext =~ /(<P_[0-9]*>\/CODE)/) {
      # Se houverem marcas de página, quando sair, ficará com a última
      $lastpg   = $1;
      $pagetext =~ s/$lastpg//;
   }

   # Insere a página e elimina o comentário
   $newoutput =~ s/( *<original *[^>]*>\/CODE(.*?)<_original>\/CODE *)/ $lastpg /;
}

# Para garantir que sempre haverá um número de página
if ($newoutput !~ /<text> <P_/) {
   $newoutput =~ s/<text> /<text> <P_0>\/CODE /g;
}

# Salta uma linha antes e depois de <text> e </text>
#$newoutput =~ s/(<text>)/\n$1\n/g;
$newoutput =~ s/(<\/text>)/| $1/g;

# Inclusão de "|" antes de página (antes de </text> foi feito acima)
$newoutput =~ s/(<P_[0-9]*>\/CODE)/ | $1/g;

# Corrige problemas com digitação e elimina tags inúteis (comment, original)
# OBS.: casos de word/TAGword/TAG tem que ser corrigido à mão
$newoutput =~ s/([^ \/]*\/[\.\(,])([^ \/]*\/[^ ]*)/$1 $2/g; # "elem/[.(,]elem/TAG"
$newoutput =~ s/([^ \/]*\/[^ \/]*)([\.;:,\-\)\(\?!]\/[^ ]*)/$1 $2/g; # "elem/TAG[.;:,-)(?!)]/TAG"

$newoutput2 = $newoutput;

#print DADOS $newoutput; die;

# Vai concatenar o texto dentro de <text></text>
$bodyoutput = "";

# Inserção das informações (processa cada ocorrência de ponto final)
print STDOUT "   Processando o corpo do texto...\n";

while ($newoutput2 =~ /(<P_([0-9]*)>\/CODE)([^|]*)\|/) {
   $bodyoutput .= " ".$1." ";

   # Page control
   $pagetext = $3; # Current matched region
   $pagenr   = $2; # Current page number
   if ($pageant != $pagenr) { # Page change
      #$sentence = 1;  # Numeração relativa à página
      $pageant  = $page;
   }

   #$bodyoutput .= " <P_".(int($pagenr)+1).">/CODE "; ## QUANDO PRECISAR SOMAR 1 À PÁGINA

   # Para facilitar a substituição, cria uma padrão único para a busca
   $pagetext =~ s/ +(([@\.!\?;:]|[@\.!\?;:]+)\/\.)/ $1PPFF /g;
   $pagetext =~ s/\s([^ \/]+)\s/ $1\/FW /g;  # Força uma etiqueta em eventuais palavras que não tenham

   # Prepara próxima inserção
   $insert = " ".$fileid.",".$pagenr.".".$sentence."/ID ./PONFP ";
   #$insert = " ".$fileid.",".(int($pagenr)+1).".".$sentence."/ID ./PONFP ";

   while ($pagetext =~ / +([@\.!\?;:]|[@\.!\?;:]+)\/\.PPFF +/) {
      # Insere na primeira ocorrência (na sentença) de ponto final
      $pagetext =~ s/ +(([@\.!\?;:]|[@\.!\?;:]+)\/\.)PPFF +/ $1$insert/;

      # Prepara próxima inserção
      $sentence++;
      $insert = " ".$fileid.",".$pagenr.".".$sentence."/ID ./PONFP ";
      #$insert = " ".$fileid.",".(int($pagenr)+1).".".$sentence."/ID ./PONFP ";
   }

   $bodyoutput .= $pagetext;

   # Retira do buffer todo o trecho relativo à página
   $newoutput2 =~ s/<P_$pagenr>\/CODE[^|]*\|//;
}

# Insere o novo corpo do texto
$newoutput =~ s/(<text>).*(<\/text>) */\n\n$1\n\n$bodyoutput$2\n/g;
$newoutput =~ s/ *(<P_[0-9]*>\/CODE)/$1\n\n/; # Primeira marca de página
$newoutput =~ s/(PONFP) +(<P_[0-9]*>\/CODE +)/$1\n\n$2\n\n/g; # Páginas restantes que ocorrem depois de PONFP
$newoutput =~ s/(CODE) +(<P_[0-9]*>\/CODE +)/$1\n\n$2\n\n/g; # Páginas restantes que ocorrem depois de /CODE
$newoutput =~ s/(PONFP)\n*/$1\n\n/g;
$newoutput =~ s/\@\/\.//g;
$newoutput =~ s/ +/ /g;
$newoutput =~ s/\n /\n/g;
$newoutput =~ s/ *(#!FORMAT=POS_0) */$1/g;

print DADOS $newoutput;
