#!/usr/bin/perl

#
# Script para inversão do elemento *pro* com os elementos CODE, *T* e CONJ, que
# ficaram em posição incorreta, após a inclusão de *pro*. Estas são alterações
# automáticas, segundo orientação de Charlotte Galves.
#
# Autor: Pablo Picasso F. de Faria
# Em 12/11/2007
#

# Verifica parâmetros
if(scalar(@ARGV) != 1) {
   print "\nScripts do Corpus Tycho Brahe, IEL/Unicamp (2008)\n";
   print "Autor: Pablo Faria (pablofaria\@gmail.com)\n\n";
   print "Uso:\n    tb-adjustPSD.pl <psdfile>\n";
   print "Saída:\n    <psdfile>.tmp\n\n";
   exit;
}

$fin  = $ARGV[0];
$fout = $ARGV[0].".tmp";

open(ARQ, "<$fin") || die;
open(DADOS1, ">$fout") || die;

# Varre o arquivo, concatenando todas as linhas em um buffer
$buffer = "";
while($linha = <ARQ>) {
   $buffer .= $linha;
}

# Esconde quebras de linha, para poder fazer certas substituições
$buffer =~ s/\n/%/g;
$buffer =~ s/%%/#/g;

# 1o. caso: *pro* seguido de CODE
$buffer =~ s/(\([0-9]* *NP-SBJ \*pro\*\))([^(]*)(\([0-9]* *CODE (.*?)\))/$3$2$1/g;

# Elimina o "CODE <comment..." tbm, pois depois será incluído o ID
$buffer =~ s/( *\([0-9]* *CODE <comm(.*?)\)([^(]*))//g;
$buffer =~ s/( *\([0-9]* *CODE <[_\$]+comm(.*?)\)([^(]*))//g;

# Elimina o "CODE <heading..." tbm
$buffer =~ s/ *\(([0-9]* *)?CODE <head(.*?)\) *//g;
$buffer =~ s/ *\(([0-9]* *)?CODE <[_\$]+head(.*?)\) *//g;

# 2o. caso: *pro* seguido de *T*
$buffer =~ s/(\([0-9]* *NP-SBJ \*(pro|exp)\*(-[0-9]+)?\))([^(]*)(\([0-9]* *[^ ]* +\*T\*\-[0-9]*\))/$5$4$1/g;

# 3o. caso: *pro* seguido de CONJ
$buffer =~ s/(\([0-9]* *NP-SBJ \*(pro|exp)\*(-[0-9]+)?\))([^(]*)(\([0-9]* *CONJ .*?\))/$5$4$1/g;

# 4o. caso: *pro* seguido de "-" (falas em textos literários)
$buffer =~ s/(\([0-9]* *NP-SBJ \*(pro|exp)\*(-[0-9]+)?\))([^(]*)(\([0-9]* *<paren> .*?\))/$5$4$1/g;

# Elimina "CODE {TEMP:", se houver
$buffer =~ s/(\([0-9]* *CODE \{TEMP:(.*?)\)([^(]*))//g;

# Substituições que devem ser feitas sem as quebras de linha
  # Retira marcas de páginas internas a uma sentença e coloca depois (início da sentença é que conta)
  $buffer =~ s/\( \(([^#]*?)[\t ]+(\(CODE <P_[0-9]+>\))(%| )([^#]*?)#/( ($1 $4#( $2)#/g;
  # Substitui $ em abertura de tags, por @, quando houver
  $buffer =~ s/\(([^ ]*) +([^ ]*?)\-\$\)(.*?)\(([^ ]*) +\$([^ ]*?)\)/($1 $2-\@)$3($4 \@$5)/g;
  $buffer =~ s/\(([^ ]*) +\$([^ ]*?)\)(.*?)\(([^ ]*) +\$([^ ]*?)\)/($1 $2\@)$3($4 \@$5)/g;

# Desgruda etiquetas e "("
$buffer =~ s/([^ #%])\(/$1 (/g;

# Normaliza
$buffer =~ s/[%#]+\)/)/g;
$buffer =~ s/\)[%# \t]+\)/))/g;

# Elimina anomalias
$buffer =~ s/\( \([^ ]+\)#//g;

# Retorna as quebras de linha
$buffer =~ s/%/\n/g;
$buffer =~ s/#/\n\n/g;

# Imprime a nova linha no arquivo
print DADOS1 $buffer;
