#!/usr/bin/perl

#
# Script Perl que realiza o processamento de um arquivo ou stream de texto,
# preparando o mesmo para etiquetagem. A preparação consiste em separar pontuações
# e sinais como áspas, parênteses, etc., das palavras. Obs.: MELHOR SE FOR UTILIZADO COM O
# XSLT "plain_tagger-v2.xsl".
#
# Ex.: "Instalação do tagger do "Fábio Kepler" (foi preciso instalar um pacote no Ubuntu).
#       O comando “vlmmtagger” executa o tagger." => "Instalação do tagger do " Fábio Kepler "
#       ( foi preciso instalar um pacote no Ubuntu ) . O comando “ vlmmtagger ” executa o
#       tagger ."
#
# Autor: Pablo Picasso F. de Faria
# Em 23/10/2007
#

# Verifica parâmetros
if(scalar(@ARGV) != 1) {
   print "\nScripts do Corpus Tycho Brahe, IEL/Unicamp (2008)\n";
   print "Autor: Pablo Faria (pablofaria\@gmail.com)\n\n";
   print "Uso:\n    tb-tagger-prep.pl <txtfile>\n";
   print "Saída:\n    <txtfile>.tmp\n\n";
   exit;
}

use locale;
use POSIX 'locale_h';

(setlocale(LC_CTYPE, 'pt_BR') || setlocale(LC_CTYPE, 'pt_BR.utf8')) or die "Invalid locale pt_BR";

$fin  = $ARGV[0];
$fout = $ARGV[0].".tmp";

open(ARQ, "<$fin") || die;
open(DADOS1, ">$fout") || die;

# Varre o arquivo, linha a linha
while($linha = <ARQ>) {

   # Separações
   $linha =~ s/:\|.*$/ /g; # Elimina comentários
   $linha =~ s/\|/ /g; # Elimina comentários
   $linha =~ s/\[.*?\]/ /g; # Elimina comentários
   $linha =~ s/\t/ /g; # Retira tabulações
   $linha =~ s/^ +//g; # Retira espaços iniciais
   $linha =~ s/< *P_([I0-9]*(v)?) *>/ <P_$1> /g; # "< P_999 >" -> <P_999>
   $linha =~ s/([0-9]+)([ªº%])/$1 $2/g; # ordinais e percentuais
   $linha =~ s/([0-9]+)([\.,])([0-9]+)/$1$2/g; # valores e números fracionados (junta, para evitar dois números após separação da vírgula, pois o valor em si, não importa para as análises)
   $linha =~ s/([áéíóúèìòùàãẽõĩũâêîôûçäëïöü\$])([="'\.;:,\)\(\?!])/$1 $2/g; # "palavra+pontuação"
   $linha =~ s/([="'\.;:,\)\(\?!])([áéíóúèìòùàãẽõĩũâêîôûçäëïöü\$])/$1 $2/g; # "pontuação+palavra"
   $linha =~ s/(\w)([="'\.;:,\)\(\?!])/$1 $2/g; # "palavra+pontuação"
   $linha =~ s/([="'\.;:,\)\(\?!])(\w)/$1 $2/g; # "pontuação+palavra"
   $linha =~ s/([\?!])\1+/$1/g; # "pontuação repetida"
   $linha =~ s/([="'\.;:,\)\(\?!])([^\1])/$1 $2/g; # "pontuação+pontuação"
   $linha =~ s/([="'\.;:,\)\(\?!])([^\1])/$1 $2/g; # "pontuação+pontuação"
   $linha =~ s/^ *(\-)(\w)/$1 $2/g; # "-Palavra" (início de fala)

   # Devem ser corrigidos na edição
   #$linha =~ s/\//<barra>/g; # "/" é para etiquetas
   #$linha =~ s/\\/<barra_inv>/g; # "\"
   $linha =~ s/ (\-)(\w)/ $1 $2/g; # " -palavra"
#    $linha =~ s/(\w)(\-) +/$1$2/g; # "palavra- resto" (erro de digitação)
   # Problema: "d'eles" deve ser modernizado para "deles"
   # Sujeiras: tudo que não tem interpretação sintática ou morfológica deve ser eliminado!!!

   $linha =~ s/ +/ /g; # Normaliza espaços
   $linha =~ s/(\. ){3,}/... /g; # Corrige reticências

   # Imprime a nova linha no arquivo
   if ($linha ne "\n") {
      # Linhas sem pontuação final, insere.
      $linha =~ s/ *\n/\n/g; # va_004
      $linha =~ s/([^\.\?!;:])\n/$1 .\n/g; # va_004

      print DADOS1 $linha;
   }
}
