#!/usr/bin/perl

#
# Script para processar um arquivo etiquetado (no formato de busca com
# CorpusSearch) e gerar uma versão 'preparse' para ser submetida ao parser.
# Se for preciso o 'split' de etiquetas, isso deve ser feito antes de rodar
# este script.
#
# Autor: Pablo Picasso F. de Faria
# Criação: 28/08/2009
#

# Verifica parâmetros
if(scalar(@ARGV) != 1) {
   print "\nScripts do Corpus Tycho Brahe, IEL/Unicamp (2008)\n";
   print "Autor: Pablo Faria (pablofaria\@gmail.com)\n\n";
   print "Uso:\n    tb-preparseFromPOS.pl <posfile>\n";
   print "Saída:\n    <posfile>.preparse\n\n";
   exit;
}

$fin  = $ARGV[0];
$fout = $ARGV[0].".preparse";

open(ARQ, "<$fin") || die;
open(DADOS, ">$fout") || die;

print STDOUT "Processando...";

# Processamento
while($linha = <ARQ>) {
   # Exclui informações para o CorpusSearch
   if($linha =~ /FORMAT/ || $linha =~ /text>/) { next; }

   $linha =~ s/ *\n/\n/;  # Remove espaços finais

   if ($linha ne "\n") {
      #$linha =~ s/(\/[^ -]+)-[^ ]+ /$1 /g;  # Retira subetiquetas
      $linha =~ s/\(/<paren>/g;   # Troca "(" por "<paren>"
      $linha =~ s/\)/<\$\$paren>/g;   # Troca ")" por "<$$paren>"
      $linha =~ s/<[_\/]/<\$\$/g;   # Troca "<_ ou </" por "<$$"
      $linha =~ s/[_\/]>/\$\$>/g;   # Troca "_> ou />" por "$$>"
      $linha =~ s/ [^ ]+\/ID \.\/PONFP//;   # Remove ID e PONFP
      $linha =~ s/^/(/g;   # Insere "(" no início da sentença
      $linha =~ s/\n/)\n\n/g;   # Insere ")" no fim da sentença e uma linha em branco
      $linha =~ s/([^ ]+)\/([^ \n]+)/($1 ($2))/g;   # Converte "word/tag" para "(word (tag))"
      $linha =~ s/\( +\(/((/g;   # Remove espaços em branco entre parênteses consecutivos
      print DADOS $linha;
   }
}

print STDOUT "Ok!\n";

close(ARQ);
close(DADOS);
