#!/usr/bin/perl

#
# Script para processar um PSD gerado na Pensilvânia, cujas tags do tipo 
# RAIZ-... vêm sem a parte "-...". Assim, com base no POS enviado para o 
# parser, pode-se recuperar estas terminações das tags do PSD.
#
# Autor: Pablo Picasso F. de Faria
# Versão final criada em 05/04/08
#

# Verifica parâmetros
if(scalar(@ARGV) != 2) {
   print "\nScripts do Corpus Tycho Brahe, IEL/Unicamp (2008)\n";
   print "Autor: Pablo Faria (pablofaria\@gmail.com)\n\n";
   print "Uso:\n    tb-retrieveSubTags.pl <posfile> <psdfile>\n";
   print "Saída:\n    <psdfile>.tmp\n\n";
   exit;
}

use locale;
use POSIX 'locale_h';

(setlocale(LC_CTYPE, 'pt_BR') || setlocale(LC_CTYPE, 'pt_BR.utf8')) or die "Invalid locale pt_BR";

# Parâmetros do prompt
$fpos = $ARGV[0];
$fpsd = $ARGV[1];
$fout = $ARGV[1].".tmp";

# Abre arquivos para leitura e arquivo de saída
open(POS, "<$fpos") || die;
open(PSD, "<$fpsd") || die;
open(DADOS1, ">$fout") || die;

# Carrega o buffer para o arquivo POS
$bufpos = "";
while($linha = <POS>) {
   $bufpos .= $linha;
}
$bufpos =~ s/\n/ /g;
#$bufpos =~ s/.*?<text>/ /g;
$bufpos =~ s/ ([^ ]*?)\/(ID|PONFP|CODE|\(|\.) / /g; # Ignora estes itens
$bufpos =~ s/ ([^ ]*?)\/(ID|PONFP|CODE|\(|\.) / /g; # Ignora estes itens
$bufpos =~ s/ ([^ ]*?)\/(ID|PONFP|CODE|\(|\.) / /g; # Ignora estes itens

# Carrega o buffer para o arquivo PSD
$bufpsd = "";
while($linha = <PSD>) {
   $bufpsd .= $linha;
}
$newpsd = $bufpsd;  # Cópia do buffer, para gerar nova versão
$newpsd =~ s/\n/%/g; # As quebras são retiradas para serem recolocadas depois
$newpsd =~ s/\t/#/g; # As tabulações são retiradas para serem recolocadas depois

# Alterações no buffer de trabalho, para poder ser processado
$bufpsd =~ s/\n/ /g;
$bufpsd =~ s/\((ID|PONFP|CODE|\.) .*?\)//g;  # Ignora estes itens

# Contagem das etiquetas corrigidas
$count = 0;

@timeData = localtime(time);
print "Início: ".@timeData[2].":".@timeData[1].":".@timeData[0]."\n";

# Consome o par a par (palavra/etiqueta), o arquivo POS
while($bufpos =~ s/ +([^\/]+)\/([^ ]+) / /) {
   # Elementos do par: palavra e etiqueta
   $wpos = $1;
   $tpos = $2;

   # Filtrar aqui (apenas etiquetas contendo "-"), e não no while acima, diminui o tempo em 75%
   if($tpos =~ /([^-]+)-[^ ]*/) {
      $tpsd = $1;
      $count++;
      #print "Word: ".$wpos.", Tag: ".$tpos."\n";
      #sleep(1);
      $newpsd =~ s/\($tpsd $wpos\)/($tpos $wpos)/;
   }
}

@timeData = localtime(time);
print "Fim: ".@timeData[2].":".@timeData[1].":".@timeData[0]."\n";

print "Etiquetas corrigidas: ".$count."\n";

$newpsd =~ s/%/\n/g; # As quebras são retiradas para serem recolocadas depois
$newpsd =~ s/#/        /g; # Recoloca tabulações convertidas para 8 espaços
print DADOS1 $newpsd;

print "Arquivo gerado: ".$fout."\n";