#!/usr/bin/perl

#
# Script para processar um PSD gerado na Pensilvânia, cujas tags do tipo
# RAIZ-... vêm sem a parte "-...". Assim, com base no POS enviado para o
# parser, pode-se recuperar estas terminações das tags do PSD.
#
# Autor: Pablo Picasso F. de Faria
# Versão final criada em 05/04/08
#

# Verifica parâmetros
if(scalar(@ARGV) != 3 ||
		($ARGV[0] ne "preparse" && $ARGV[0] ne "psd") ||
		($ARGV[1] ne "under" && $ARGV[1] ne "hifen")) {
   print "\nScripts do Corpus Tycho Brahe, IEL/Unicamp (2008)\n";
   print "Autor: Pablo Faria (pablofaria\@gmail.com)\n\n";
   print "Uso:\n    tb-underscoreSubTags.pl preparse|psd under|hifen <file>\n";
   print "Saída:\n    <file>.tmp\n\n";
   exit;
}

use locale;
use POSIX 'locale_h';

(setlocale(LC_CTYPE, 'pt_BR') || setlocale(LC_CTYPE, 'pt_BR.utf8')) or die "Invalid locale pt_BR";

# Parâmetros do prompt
$fpsd = $ARGV[2];
$fout = $ARGV[2].".tmp";

# Abre arquivos para leitura e arquivo de saída
open(PSD, "<$fpsd") || die;
open(DADOS1, ">$fout") || die;

# Carrega o buffer para o arquivo PSD
$bufpsd = "";
while($linha = <PSD>) {
   $bufpsd .= $linha;
}
$newpsd = $bufpsd;  # Cópia do buffer, para gerar nova versão
$newpsd =~ s/\n/%/g; # As quebras são retiradas para serem recolocadas depois
$newpsd =~ s/\t/#/g; # As tabulações são retiradas para serem recolocadas depois

# Alterações no buffer de trabalho, para poder ser processado
$bufpsd =~ s/\n/ /g;

@timeData = localtime(time);
print "Início: ".@timeData[2].":".@timeData[1].":".@timeData[0]."\n";

if ($ARGV[0] eq "psd") {
	# PSD file
	if ($ARGV[1] eq "under") {
		 # UNDER(SCORE): para preparar o arquivo de treinamento do parser
		 $newpsd =~ s/([^ ]*?)-[0-9]+/$1/g;  # Remove índices
		 while ($newpsd =~ /(\([^ (]*?)-[^ ]+ /) {
				$newpsd =~ s/(\([^ (]*?)-([^ ]+) /$1_$2 /g;  # Substitui '-' por '_'
		 }
	} else {
		 # HIFEN(IZE): Usado para corrigir o arquivo PSD gerado
		 while ($newpsd =~ /(\([^ (]*?)_[^ ]+ /) {
				$newpsd =~ s/(\([^ (]*?)_([^ ]+) /$1-$2 /g;  # Substitui '_' por '-'
		 }
	}
} else {
	# Preparse file
	if ($ARGV[1] eq "under") {
		 # UNDER(SCORE): para preparar o arquivo a enviar ao parser
		 while ($newpsd =~ / (\([^ (]*?)-[^ \)]+\)/) {
				$newpsd =~ s/ (\([^ (]*?)-([^ \)]+\))/ $1_$2/g;  # Substitui '-' por '_'
		 }
	} else {
		 # HIFEN(IZE): Usado para corrigir o arquivo preparse
		 while ($newpsd =~ / (\([^ (]*?)_[^ \)]+\)/) {
				$newpsd =~ s/ (\([^ (]*?)_([^ \)]+\))/ $1-$2/g;  # Substitui '_' por '-'
		 }
	}
}

@timeData = localtime(time);
print "Fim: ".@timeData[2].":".@timeData[1].":".@timeData[0]."\n";

$newpsd =~ s/%/\n/g; # As quebras são retiradas para serem recolocadas depois
$newpsd =~ s/#/        /g; # Recoloca tabulações convertidas para 8 espaços
print DADOS1 $newpsd;

print "Arquivo gerado: ".$fout."\n";